@echo off
rem  @(#)77    1.14  src/invscan/w32/mbsa/MBSASCAN.CMD, invscan, cit_columbus_1, 20061016v111_1  4/4/05  02:27:05
rem
rem Licensed Materials - Property of IBM
rem
rem 5698-INV
rem
rem (C) Copyright IBM Corp. 2002, 2002 All Rights Reserved
rem
rem US Government Users Restricted Rights - Use, duplication or
rem disclosure restricted by GSA ADP Schedule Contract with
rem IBM Corp
rem

rem +----------------------------------------------------+
rem |                                                    |
rem | Script: mbsascan.cmd                               |
rem | Version: 1.7                                       |
rem | Author: Damir Bacalja                              |
rem | Modified by: Li Lin                                | 
rem | Description: This script runs MBSACLI.EXE program  |
rem | and parses its output through mbsaparser.exe       |
rem |                                                    |
rem +----------------------------------------------------+

rem
rem Script gets kicked with parameter setenv from Inventory
rem profile. Then it restarts using cmd /x /v:on /c parameters.
rem This is to make sure command extensions and delayed 
rem environment variable expansion is turned on (by default 
rem it is always off).
rem

set ERRORLEVEL=0
set SCANOPT=""
set OUTPUT="mif"
set DEBUG=""
set ERFLAG=""
set PROD="false"
set SOPT="false"
set APPfile=""
set APPdir=""
if "%1"=="" (
  goto Usage
) else (
  set INPUT=%1
  call :SetInput
)

rem echo %ERFLAG%
if %ERFLAG%=="true" (
  goto Usage
)

if "%2"=="" (
  goto Start
)else (
  set INPUT=%2
  call :SetInput
)

rem echo %ERFLAG%
if %ERFLAG%=="true" (
  goto Usage
)

if "%3"=="" (
  goto Start
)else (
  set INPUT=%3
  call :SetInput
)

if "%4"=="" (
  goto Start
)else (
  set INPUT=%4
  call :SetInput
)

rem echo %ERFLAG%
if %ERFLAG%=="true" (
  goto Usage
)

if not "%5"==""  (
  goto Usage
)
  
rem echo OPTION: %SCANOPT%
rem echo OUTPUT: %OUTPUT%

goto Start

:SetInput
rem echo setinput
rem echo INPUT:%INPUT%
if %INPUT%==-i (
  set SCANOPT="1"
) else if %INPUT%==-I (
  set SCANOPT="1"
) else if %INPUT%==-n (
  set SCANOPT="2"
) else if %INPUT%==-N (
  set SCANOPT="2"
) else if %INPUT%==-a (
  set SCANOPT="3"
) else if %INPUT%==-A (
  set SCANOPT="3"
) else if %INPUT%==-d (
  set DEBUG="-d"
) else if %INPUT%==-D (
  set DEBUG="-d"
) else if %INPUT%==-x (
  set OUTPUT="xml"
) else if %INPUT%==-X (
  set OUTPUT="xml"
) else if %INPUT%==-p (
  set PROD="true"
) else if %INPUT%==-s (
  set SOPT="true"
) else (
  set ERRMSG=ERROR: Don't have this option: "%INPUT%".
  set ERFLAG="true"
)
goto :EOF

:Usage
  echo Usage: MBSASCAN {-i ^| -n ^| -a} [-x] [-d] [-p]
  echo                  -i: scan for patches that have been explicitly installed.
  echo                  -n: scan for patches that have been explicitly not installed.
  echo                  -a: scan for patches that have been effectively installed
  echo                  -x: output in XML format. otherwise will be MIF format.
  echo                  -d: debug enabled.
  echo                  -p: populate the product id field.
  echo                  -s: look for ApprovedItem.txt file.
goto LogErr

:LogErr
set LOGfile="mbsascan.log"
set LOGfilebk="mbsascan.log.bk"
if exist %LOGfile% (
  if exist %LOGfilebk% (
    del %LOGfilebk%
  ) 
  rename %LOGfile% %LOGfilebk%
)
echo %ERRMSG% > %LOGfile%
echo %ERRMSG% 1>&2
set ERRORLEVEL=1
goto End

:Start
rem
rem This script starts the MBSA scan and generates
rem MIF file from the MBSA tool output.
rem
rem It was originally written in VBS, then I rewrote it
rem as shell script and now as CMD file. Hopefully this
rem is it's final format. 8-)
rem

rem echo SCANOPT:%SCANOPT%
if %SCANOPT%=="" (
  set ERRMSG=ERROR: one of the SCAN OPTIONs -i, -n or -a has to be chosen.
  goto Usage
)

rem set working directory
for /f "tokens=*" %%i in ('cd') do set WORKDIR=%%i
rem echo workdir is %WORKDIR%

if not exist "%WORKDIR%" (
  set ERRMSG=ERROR: %WORKDIR% does not exist.
  goto LogErr
)

set MIFfile="%WORKDIR%\mbsascan.mif"
set CABfile="%WORKDIR%\mssecure.cab"
set MBSAPARSER="%WORKDIR%\mbsaparser.exe"
set TMPfile="%WORKDIR%\mbsacli.out"

rem
rem These 4 variables are paths where to search for MBSA tool
rem
set MBSADIR0="C:\Program Files\Microsoft Baseline Security Analyzer"
set MBSADIR1="%ProgramFiles%\Microsoft Baseline Security Analyzer"
set MBSADIR2="C:\MBSA"
set MBSADIR3="%ProgramFiles%\MBSA"
set MBSADIR4="."

if exist %MBSADIR0%\mbsacli.exe (
    set MBSASCANNER=%MBSADIR0%\mbsacli.exe
    set XMLfile=%MBSADIR0%\mssecure.xml
    set APPdir=%MBSADIR0%
) else if exist %MBSADIR1%\mbsacli.exe (
    set MBSASCANNER=%MBSADIR1%\mbsacli.exe
    set XMLfile=%MBSADIR1%\mssecure.xml
    set APPdir=%MBSADIR1%
) else if exist %MBSADIR2%\mbsacli.exe (
    set MBSASCANNER=%MBSADIR2%\mbsacli.exe
    set XMLfile=%MBSADIR2%\mssecure.xml
    set APPdir=%MBSADIR2%
) else if exist %MBSADIR3%\mbsacli.exe (
    set MBSASCANNER=%MBSADIR3%\mbsacli.exe
    set XMLfile=%MBSADIR3%\mssecure.xml
    set APPdir=%MBSADIR3%
) else if exist %MBSADIR4%\mbsacli.exe (
    set MBSASCANNER=%MBSADIR4%\mbsacli.exe
    set XMLfile=%MBSADIR4%\mssecure.xml
    set APPdir=%MBSADIR4%
) else (
    set ERRMSG=ERROR: Could not find MBSA scanner.
    goto LogErr
)

if %SOPT%=="true" (
	if exist %APPdir%\Approveditems.txt (
		set APPfile=%APPdir%\Approveditems.txt
      )
)


rem %MBSASCANNER% /hf -history %SCANOPT% /x %CABfile% /o tab | findstr /vi "Q Number" > %TMPfile%

echo APPfile value is %APPfile%

if %APPfile% == "" (
	echo calling mbsacli without APPfile .......
	%MBSASCANNER% /hf -history %SCANOPT% /x %CABfile% /o tab /nvc > %TMPfile%
)else (
	echo calling mbsacli with APPfile ......
	%MBSASCANNER% /hf -history %SCANOPT% /x %CABfile% /o tab /nvc /sus %APPfile% > %TMPfile%
)

rem
rem Parse MBSA scan results
rem

if not exist %TMPfile% (
  set ERRMSG=ERROR: %TMPfile% does not exist.
  goto LogErr
)

if %OUTPUT%=="xml"  (
  if %DEBUG%=="" (
	if %PROD%=="true" (
	    %MBSAPARSER% -i %TMPfile% -x -p
	) else (
    	    %MBSAPARSER% -i %TMPfile% -x
	)
  ) else (
	if %PROD%=="true" (
	    %MBSAPARSER% -i %TMPfile% -x -d -p
	) else (
	    %MBSAPARSER% -i %TMPfile% -x -d
  	)
  )
) else (
  if %DEBUG%=="" (
	if %PROD%=="true" (
	    %MBSAPARSER% -i %TMPfile% -p 
	) else ( 
    	    %MBSAPARSER% -i %TMPfile% 
	)
  ) else (
	if %PROD%=="true" (
	    %MBSAPARSER% -i %TMPfile% -d -p
	) else (
    	    %MBSAPARSER% -i %TMPfile% -d 
	)
    )
 )
)

if exist %XMLfile% del %XMLfile%
if exist %TMPfile% del %TMPfile%

:End

