' @(#)72  1.13  src/hc/client/collectors/C/vbs/groups.vbs, hc.collectors.win, scmweb  7/15/11  08:11:49
'
' IBM Security Compliance Manager
' 5724-F82
' (C) IBM, 2006.
' All Rights Reserved
' Licensed Materials - Property of IBM
'
' US Government Users Restricted Rights - Use, duplication or
' disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
'
'------------------------------------------------------------

Const ADS_GROUP_TYPE_BUILTIN = &H1
Const ADS_GROUP_TYPE_GLOBAL_GROUP = &h2
Const ADS_GROUP_TYPE_LOCAL_GROUP = &h4
Const ADS_GROUP_TYPE_UNIVERSAL_GROUP = &h8
Const ADS_GROUP_TYPE_SECURITY_ENABLED = &h80000000

Const ADS_NAME_INITTYPE_GC = 3
Const ADS_NAME_TYPE_NT4 = 3
Const ADS_NAME_TYPE_1779 = 1
Const ADS_NAME_TYPE_SID_OR_SID_HISTORY_NAME = 12

Const GROUP_SCOPE_LOCAL = "local"
Const GROUP_SCOPE_GLOBAL = "global"
Const GROUP_SCOPE_UNIVERSAL = "universal"
Const GROUP_SCOPE_UNKNOWN = "unknown"
Const GROUP_SCOPE_BUILTIN = "builtin"
Const GROUP_TYPE_SECURITY = "security"
Const GROUP_TYPE_DISTRIBUTION = "distribution"
Const GROUP_TYPE_UNKNOWN = NULL

Const ROLE_STANDALONE_WST = 0
Const ROLE_MEMBER_WST = 1
Const ROLE_STANDALONE_SVR = 2
Const ROLE_MEMBER_SVR = 3
Const ROLE_BACKUP_DC = 4
Const ROLE_PRIMARY_DC = 5

On Error Resume Next


'------------------------------------------------------------
' Main
'------------------------------------------------------------
root = NULL
domain = NULL
isActiveDirectory = FALSE
localOnBDC = FALSE
wmiDomainName = NULL

'------------------------------------------------------------
' Parse (optional) script argument(s)
'------------------------------------------------------------
For Each arg in WScript.Arguments
  If arg = "localOnBDC" Then
    localOnBDC = TRUE
  End If
Next
WScript.Echo "# localOnBDC = " & localOnBDC

Set allAccountsDict = CreateObject("Scripting.Dictionary") 

Set wmi = GetObject("winmgmts:\\.\root\cimv2")
Set compList = wmi.ExecQuery("Select DomainRole, Name, Domain from Win32_ComputerSystem")
For Each computer in compList
	computerRole = computer.DomainRole
	computerName = computer.Name
	wmiDomainName = computer.Domain
	WScript.Echo "# computerName = " & computerName
	WScript.Echo "# computerRole = " & computerRole
	WScript.Echo "# domainName = " & wmiDomainName
Next
' Decide which way of resolving use on this machine
If (computerRole = ROLE_PRIMARY_DC) OR ((computerRole = ROLE_BACKUP_DC) AND NOT localOnBDC) Then
	'Active Directory
	Set root = GetObject("LDAP://RootDSE")
	Set domain = GetObject("LDAP://" & root.Get("dnsHostName"))
	If (Not IsNull(root)) AND (Not IsNull(domain)) Then
		isActiveDirectory = TRUE
		WScript.Echo "# isActiveDirectory = " & isActiveDirectory
		Call ListForActiveDirectory(domain)
	Else
		WScript.StdErr.Write "[Error] Domain controller is not responding."
		WScript.Quit(-1)
	End If
Else
	'Local computer or Backup Domain Controller and we are not interested in domain groups
	isActiveDirectory = FALSE
	WScript.Echo "# isActiveDirectory = " & isActiveDirectory
    If ( (computerRole = ROLE_STANDALONE_WST) OR (computerRole = ROLE_STANDALONE_SVR) ) Then
		Call ListForLocalComputer(computerName, computerRole, "")
	Else
		Call ListForLocalComputer(computerName, computerRole, "")
	End If

	
End If

WScript.Quit(0)

'------------------------------------------------------------
' Lists groups and members for local computer, 
' ie. non-domain controller station
'------------------------------------------------------------
Sub ListForLocalComputer(computerName, computerRole, querySuffix)
    ' Prepare WMI
	Set rss = GetObject("WinNT://" & computerName & "")
	
	If ((computerRole = ROLE_BACKUP_DC) AND localOnBDC) Then
	 ' on Backup Domain Controller, list domain local groups
	 ' See MSDN: ADSI Objects of WinNT, http://msdn.microsoft.com/en-us/library/aa772211(VS.85).aspx
	 rss.Filter = Array("localgroup")
	Else
	 rss.Filter = Array("group")
	End If
	For Each rsItem in rss
		Set obj = CreateObject("Scripting.Dictionary") 
        groupSid = ConvertSidToString(rsItem.objectSid)
		
		' add current account to list
		allAccountsDict.Add groupSid, groupSid
			
		obj.Add "cn", rsItem.Name
        obj.Add "objectSid", groupSid
        obj.Add "groupType", rsItem.groupType
        obj.Add "description", rsItem.description
        obj.Add "distinguishedName", rsItem.Name  

        DisplayGroupInfo obj
		
        For Each asItem In rsItem.Members
			DisplayMemberInfo asItem, groupSid
        Next
            
        WScript.Echo  
    Next

	' There might be accounts without parent group so must be listed
	rss.Filter = Array("user")
    For Each rsItem in rss
        itemSid = ConvertSidToString(rsItem.objectSid)
		
		If Not allAccountsDict.Exists(itemSid) Then
			DisplayMemberInfoExt rsItem, "", "leaf"
		End If
	Next

    WScript.Echo  
	
End Sub

'------------------------------------------------------------
' Lists objects for Domain Controller station
'------------------------------------------------------------
Sub ListForActiveDirectory(domain)
	Dim OPWin2003
	OPWin2003 = "2003"
    ' Prepare ADODB
    Set con = CreateObject("ADODB.Connection")
    Set cmd = CreateObject("ADODB.Command")
    con.Provider = "ADsDSOObject"
    con.Open "Active Directory Provider"
    cmd.ActiveConnection = con
	cmd.Properties("Timeout") = 3
	cmd.Properties("Cache Results") = False
	
	' check if SO is 2003, for some reason on 2003 
	' cmd.Properties("Page Size") = 1000 causes CPU usage 
	If InStr(GetOSVersion() , OPWin2003 ) = 0 Then
    	cmd.Properties("Page Size") = 1000
	End If
	
	
	domainControllerDomain = GetDomainNameFromDN(domain.distinguishedName)
	
	' Domain dictionary
	Set objDomainList = CreateObject("Scripting.Dictionary")
	objDomainList.CompareMode = vbTextCompare
	
	' First: containers in current domain
	domain.Filter = Array("container","builtinDomain")
	For Each objContainer In domain

		' Second: groups in container
		objContainer.Filter = Array("group")
		For Each objGroup In objContainer

			' Prepare group object to be displayed
			Set groupDict = CreateObject("Scripting.Dictionary") 
			PrepareObject groupDict, objGroup
			DisplayGroupInfo groupDict
		
			groupSid = ConvertSidToString(objGroup.objectSid)

			' ---------------------------------
			sADsPath = "<" & domain.ADsPath & ">"
			strFilter = "(sAMAccountName=" &  GetSingleNameFromSid( groupSid ) & ")" 
			strAttributes = "member"
			strQuery = sADsPath & ";" & strFilter & ";" & strAttributes & ";subtree" 
'			WScript.Echo strQuery
			cmd.CommandText = strQuery 
			Set adoRecordset = cmd.Execute 
 
			Do Until adoRecordset.EOF
				For Each objField In adoRecordset.Fields
					If (VarType(objField) = (vbArray + vbVariant)) Then
						For Each strDN In objField.Value
							mm = NULL
							dnDomain = GetDomainNameFromDN(strDN)
							
							'If (objDomainList.Exists(dnDomain) = False) Then
							IF (dnDomain = domainControllerDomain) Then
								On Error Resume Next
								Set mm = GetObject("LDAP://" & strDN )
							End If
							
							If ( IsNull(mm) ) Then
								' add to dictionary
								objDomainList.Add dnDomain, True
								
								Set objTrans = CreateObject("NameTranslate")
								objTrans.Init ADS_NAME_INITTYPE_GC, ""
								objTrans.Set ADS_NAME_TYPE_1779, strDN
								strUserNT = objTrans.Get(ADS_NAME_TYPE_NT4)
								strUserSid = objTrans.Get(12)
								'GetSidFromName(strUserNT)

								WScript.Echo 
								WScript.Echo "[member]"
								Wscript.Echo "name         = " & strUserNT
								WScript.Echo "itemType     = " & ""
								WScript.Echo "itemSID      = " & strUserSid
								WScript.Echo "parentSID    = " & groupSid
								WScript.Echo "dName        = " & strDN
							
							Else 
								DisplayMemberInfo mm, groupSid
							End If
						Next
					ElseIf (VarType(objField) <> (vbNull)) Then
						'WScript.Echo ConvertSidToString(objField)
					End If
				
				Next
				adoRecordset.MoveNext
			Loop
			adoRecordset.Close
			
			' ---------------------------------
			
			' Third.One: members of a group
'			For Each objMember In objGroup.Members
'				DisplayMemberInfo objMember, groupSid
'			Next

			' Third.Two: primary group members
			objGroup.GetInfoEx Array("primaryGroupToken"), 0
			
			sADsPath = "<" & domain.ADsPath & ">"
			strAttributes = "distinguishedName,sAMAccountName,objectCategory,objectSid" 
			strFilter = "((primaryGroupID=" & objGroup.Get("primaryGroupToken") & "))"
			strQuery = sADsPath & ";" & strFilter & ";" & strAttributes & ";subtree" 
			cmd.CommandText = strQuery 
			Set rsMembers = cmd.Execute 

			Do Until rsMembers.EOF 
				mm = NULL
				Set mm = GetObject("LDAP://" & rsMembers.Fields("distinguishedName").Value )
				DisplayMemberInfo mm, groupSid
				rsMembers.MoveNext 
			Loop
		
			WScript.Echo 
			
			Set groupDict = Nothing
			Set groupSid = Nothing
			Set rsMembers = Nothing
			Set mm = Nothing
		Next

	Next
End Sub

Function GetDomainNameFromDN(dn)
	start = InStr(UCase(dn), "DC=")
	'WScript.Echo start &" "& dn &"|"& Right(UCase(dn), Len(dn)-start+1)
	GetDomainNameFromDN = Right(UCase(dn), Len(dn)-start+1)
End Function

'------------------------------------------------------------
' Returns domain and name based on given SID
'------------------------------------------------------------
Function GetSidFromName(name) 
	n = Split(name, "\")
	WScript.Echo ">"& n(0) &"<>"& n(1) &"<"
	strComputer = "."
	Set wmiService = GetObject("winmgmts:\\" & strComputer & "\root\cimv2")  
'	qry = "SELECT SID,Name FROM Win32_Account WHERE (Domain='"& n(0) &"' AND Name='"& n(1) &"')"
'	Set objList = wmiService.ExecQuery(qry)
'	For Each a in objList
'		tmp = a.SID
'	Next
	qry = "Win32_Account.Domain='" & n(0) & "',Name='" & n(1) & "'"
	WScript.Echo "@@ QRY: " & qry
	Set acc = wmiService.Get(qry)
	WScript.Echo "@@ ACC" 
	WScript.Echo "@@ SID" & acc.SID
	GetSidFromName = "!@@"
End Function

'------------------------------------------------------------
' Returns domain name based on given SID 
'------------------------------------------------------------
Function GetDomainNameFromSid(objectSid) 
	Set wmiService = GetObject("winmgmts:\\.\root\cimv2")  
	arr = Split(objectSid, "-")
	ReDim Preserve arr(UBound(arr) - 1)
	domainSid = Join(arr, "-")
	Set obj = wmiService.Get("Win32_SID.SID='" & domainSid & "'")  
	GetDomainNameFromSid = obj.ReferencedDomainName
End Function

'------------------------------------------------------------
' Returns domain and name based on given SID
'------------------------------------------------------------
Function GetNameFromSid(sid) 
	strComputer = "."
	Set wmiService = GetObject("winmgmts:\\" & strComputer & "\root\cimv2")  
	Set obj = wmiService.Get("Win32_SID.SID='" & sid & "'")  
	strName = obj.AccountName  
	strDomain = obj.ReferencedDomainName 
	GetNameFromSid = strDomain & "\" & strName
End Function

'------------------------------------------------------------
' Returns single name based on given SID
'------------------------------------------------------------
Function GetSingleNameFromSid(sid) 
	strComputer = "."
	Set wmiService = GetObject("winmgmts:\\" & strComputer & "\root\cimv2")  
	Set obj = wmiService.Get("Win32_SID.SID='" & sid & "'")  
	GetSingleNameFromSid = obj.AccountName  
End Function

'------------------------------------------------------------
' Displays information about group member
'------------------------------------------------------------
Sub DisplayMemberInfo(member, groupSid) 
	DisplayMemberInfoExt member, groupSid, "member"
End Sub

Sub DisplayMemberInfoExt(member, groupSid, nodeName) 
	On Error Resume Next 
    
	mType = NULL
	mDecSid = NULL
	name = NULL

	mType = LCase(member.Class)
	mObjSid = member.Get("objectSid")
	If (isActiveDirectory) Then
		dn = member.get("distinguishedName")
	End If
	
	If Err.Number <> 0 Then
		name = member.Get("name")
	Else 
		mDecSid = ConvertSidToString( mObjSid )
		name = GetNameFromSid( mDecSid )
		If (name="\") Then
		    If (isActiveDirectory) Then
				name = GetDomainNameFromSid( mDecSid ) & "\" & member.sAMAccountName
			Else
				name = mDecSid
				mType = "unknown"
			End If
		End If
	End If
	Err.Clear
	
	' don't display and add groups because they will be processed by parent loop
	If Not (mType = "group") Then
		WScript.Echo
		WScript.Echo "[" & nodeName & "]"
		Wscript.Echo "name         = " & name
		WScript.Echo "itemType     = " & mType
		WScript.Echo "itemSID      = " & mDecSid
		WScript.Echo "parentSID    = " & groupSid
		WScript.Echo "dName        = " & dn
		' add current account to list
		If Not allAccountsDict.Exists(mDecSid) Then
			allAccountsDict.Add mDecSid, mDecSid
		End If
	End If
		
	Set mType = Nothing
	Set mName = Nothing
	Set mDecSid = Nothing
    
End Sub


'------------------------------------------------------------
' Displays group scope
'------------------------------------------------------------
Function GetGroupScope(val)
	scope = GROUP_SCOPE_UNKNOWN

    If val AND ADS_GROUP_TYPE_LOCAL_GROUP Then 
        scope = GROUP_SCOPE_LOCAL
    ElseIf val AND ADS_GROUP_TYPE_GLOBAL_GROUP Then
        scope = GROUP_SCOPE_GLOBAL
    ElseIf val And ADS_GROUP_TYPE_UNIVERSAL_GROUP Then
        scope = GROUP_SCOPE_UNIVERSAL
    End If
	
	If val AND ADS_GROUP_TYPE_BUILTIN Then
		If scope = GROUP_SCOPE_UNKNOWN Then 
			scope = GROUP_SCOPE_BUILTIN
		Else 
			scope = scope & " " & GROUP_SCOPE_BUILTIN
		End If
	End If

	GetGroupScope = scope
	
End Function


'------------------------------------------------------------
' Transforms record object into simple hashmap. Input 'record'
' parameter type depends on platform and workstation type
'------------------------------------------------------------
Sub PrepareObject(obj, record)
    
	If (TypeName(record) = "Object") Then
        obj.Add "cn", record.cn
        obj.Add "objectSid", ConvertSidToString( record.objectSid )
        obj.Add "groupType", record.groupType
        obj.Add "description", record.description
        obj.Add "distinguishedName", record.distinguishedName 
        obj.Add "primaryGroupToken", record.primaryGroupToken  
        
    ElseIf ( (TypeName(record) = "SWbemObjectEx") OR (TypeName(record) = "ISWbemObject") ) Then
		' ISWbemObject for Windows 2000
		' SWbemObjectEx for Windows XP, 2003 R2
        obj.Add "cn", record.Properties_.Item("Name").Value
        obj.Add "objectSid", record.Properties_.Item("SID").Value
        obj.Add "groupType", ADS_GROUP_TYPE_LOCAL_GROUP
        obj.Add "description", record.Properties_.Item("Description").Value
        obj.Add "distinguishedName", record.Properties_.Item("Name").Value  
       
    End If

End Sub

 
'------------------------------------------------------------
' Displays group information nodes
'------------------------------------------------------------
Sub DisplayGroupInfo(object)
    WScript.Echo "[group]"
    
    ' name
    Wscript.Echo "name         = " & GetNameFromSid( object.Item("objectSid") ) 
	
    ' scope
    val = object.Item("groupType")
    groupScope = NULL
    If (isActiveDirectory OR ((computerRole = ROLE_BACKUP_DC) AND localOnBDC)) Then
	    groupScope = GetGroupScope(val)
	End If
   	WScript.Echo "scope        = " & groupScope
    
    groupType = NULL
    If (isActiveDirectory OR ((computerRole = ROLE_BACKUP_DC) AND localOnBDC)) Then
        If val And ADS_GROUP_TYPE_SECURITY_ENABLED Then
            groupType = GROUP_TYPE_SECURITY
        Else
            groupType = GROUP_TYPE_DISTRIBUTION
        End If
    End If
    WScript.Echo "type         = " & groupType
  
    ' comment   
    comment = NULL
    descr =  object.Item("description")
    If not IsNull(descr) Then
		comment = descr
    End If
    WScript.Echo "comment      = " & comment

    ' sid
    decSid = NULL
    decSid = object.Item("objectSid")
    WScript.Echo "groupSID     = " & decSid

    ' domain
    domainName = NULL
    If (isActiveDirectory) Then
        domainName = domain.Get("Name")
    ElseIf ((computerRole = ROLE_BACKUP_DC) AND localOnBDC) Then
        domainName = wmiDomainName
    End If
    WScript.Echo "domainName   = " & domainName

    ' distinguished name
    dn = NULL
    If (isActiveDirectory OR ((computerRole = ROLE_BACKUP_DC) AND localOnBDC)) Then
        dn = object.Item("distinguishedName")
    End If
    WScript.Echo "dName        = " & dn
    
End Sub

'------------------------------------------------------------
' Converts binary SID into string
'------------------------------------------------------------
' Structure of an SID is as follows: 
'   1 1 1 1 1 1 
'   5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 
' +---------------------------------+ 
' | SubAuthCount   |Re(SBZ)|  Rev   | 
' +---------------------------------+ 
' |     IdentifierAuthority[0]      | 
' +---------------------------------+ 
' |     IdentifierAuthority[1]      | 
' +---------------------------------+ 
' |     IdentifierAuthority[2]      | 
' +---------------------------------+ 
' |                                 | 
' + -  -  -  SubAuthority[]  -  - - + 
' |                                 | 
' +---------------------------------+
Function ConvertSidToString(sid)
	revision = Ascb(Midb(sid, 1, 1))
	subAuthCount = CInt(Ascb(Midb(sid, 2, 1)))
	
    Dim sidauth(5)
	For i = 0 To 5
	    sidauth(i) = Ascb(Midb(sid, 3+i, 1))
    Next
	If ( (sidauth(0) <> 0) OR (sidauth(1) <> 0) ) Then
	    authority = "0x"
	    For i = 0 To 5
	        authority = authority & Right("0" & Hex(Ascb(sidauth(i))), 1)
        Next       
	Else
	    authority = CLng(sidauth(2)*(256^3) + sidauth(3)*(256^3) _
                         + sidauth(4)*(256^1) + sidauth(5))
	End If

    strSid = "S-" & revision & "-" & authority
	
    c = 9
	For i = 1 To subAuthCount
        auth = 0
        For j = 0 To 3
	        auth = auth + CLng(Ascb(Midb(sid,c,1)))*(256^j)
            c = c + 1
        Next
	    strSid = strSid & "-" & cstr(auth)
    Next
	ConvertSidToString = strSid
End Function

'------------------------------------------------------------
' Function that return the OS Version
' Ex. Microsoft(R) Windows(R) Server 2003, Enterprise Edition
'------------------------------------------------------------
Function GetOSVersion()
On Error Resume Next
	Dim OPName
	strComputer = "."
	Set objWMIService = GetObject("winmgmts:\\" & strComputer & "\root\cimv2")
	
	Set colOperatingSystems = objWMIService.ExecQuery _
	    ("Select * from Win32_OperatingSystem")
	
	
	For Each objOperatingSystem in colOperatingSystems
	    OPName = objOperatingSystem.Caption
	Next
	
	GetOSVersion = OPName
End Function

